/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.keycloak.representations.idm.RealmRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.operator.keycloak.realm.OzgCloudKeycloakRealmSpec.KeycloakRealmSMTPServer;

@Mapper(unmappedTargetPolicy = ReportingPolicy.IGNORE, unmappedSourcePolicy = ReportingPolicy.IGNORE)
interface KeycloakRealmMapper {

	public static final String ACTION_TOKEN_GENERATED_BY_USER_LIFE_SPAN = "900";
	public static final String PASSWORD_POLICY = "upperCase(1) and lowerCase(1) and length(8) and notUsername";
	public static final String DEFAULT_LOCALE = "de";
	public static final String RESET_PASSWORD_ALLOWED = "true";
	public static final String ENABLED = "true";
	public static final String INTERNATIONALIZATION_ENABLED = "true";

	@Mapping(target = "enabled", constant = ENABLED)
	@Mapping(target = "resetPasswordAllowed", constant = RESET_PASSWORD_ALLOWED)
	@Mapping(target = "supportedLocales", source = ".", qualifiedByName = "supportedLocales")
	@Mapping(target = "defaultLocale", constant = DEFAULT_LOCALE)
	@Mapping(target = "internationalizationEnabled", constant = INTERNATIONALIZATION_ENABLED)
	@Mapping(target = "passwordPolicy", constant = PASSWORD_POLICY)
	@Mapping(target = "actionTokenGeneratedByUserLifespan", constant = ACTION_TOKEN_GENERATED_BY_USER_LIFE_SPAN)
	@Mapping(target = "smtpServer", source = "smtpServer", qualifiedByName = "smtpServer")
	@Mapping(target = "roles.realm", source = "realmRoles")
	public RealmRepresentation map(OzgCloudKeycloakRealmSpec realm);

	@Mapping(target = "enabled", constant = ENABLED)
	@Mapping(target = "resetPasswordAllowed", constant = RESET_PASSWORD_ALLOWED)
	@Mapping(target = "supportedLocales", source = ".", qualifiedByName = "supportedLocales")
	@Mapping(target = "defaultLocale", constant = DEFAULT_LOCALE)
	@Mapping(target = "internationalizationEnabled", constant = INTERNATIONALIZATION_ENABLED)
	@Mapping(target = "passwordPolicy", constant = PASSWORD_POLICY)
	@Mapping(target = "actionTokenGeneratedByUserLifespan", constant = ACTION_TOKEN_GENERATED_BY_USER_LIFE_SPAN)
	@Mapping(target = "smtpServer", source = "smtpServer", qualifiedByName = "smtpServer")
	@Mapping(target = "roles.realm", source = "realmRoles")
	RealmRepresentation update(@MappingTarget RealmRepresentation existingRealm, OzgCloudKeycloakRealmSpec spec);

	@Mapping(target = "name", source = "name")
	RoleRepresentation map(OzgCloudKeycloakRealmSpec.RealmRole role);

	@Named("supportedLocales")
	default Set<String> mapPassword(OzgCloudKeycloakRealmSpec spec) {
		return Set.of("de");
	}

	@Named("smtpServer")
	default Map<String, String> mapSmtpServer(OzgCloudKeycloakRealmSpec.KeycloakRealmSMTPServer server) {

		Map<String, String> smtpServer = new HashMap<>();
		if (server != null) {
			smtpServer.put(KeycloakRealmSMTPServer.HOST_FIELD, server.getHost());
			smtpServer.put(KeycloakRealmSMTPServer.PORT_FIELD, server.getPort());
			smtpServer.put(KeycloakRealmSMTPServer.USER_FIELD, server.getUser());
			smtpServer.put(KeycloakRealmSMTPServer.PASSWORD_FIELD, server.getPassword());
			smtpServer.put(KeycloakRealmSMTPServer.STARTTLS_FIELD, server.getStarttls());
			smtpServer.put(KeycloakRealmSMTPServer.AUTH_FIELD, server.getAuth());
			smtpServer.put(KeycloakRealmSMTPServer.FROM_FIELD, server.getFrom());
			smtpServer.put(KeycloakRealmSMTPServer.FROM_DISPLAY_NAME_FIELD, server.getFromDisplayName());
		}
		return smtpServer;

	}

}
